import 'dart:html';

import 'package:utilary/utilary.dart' show Vector;

abstract class Input {
  final bool shift;
  final bool alt;
  final bool ctrl;

  const Input({bool shift = false, bool alt = false, bool ctrl = false})
      : this.shift = shift,
        this.alt = alt,
        this.ctrl = ctrl;

  bool operator ==(Object o) =>
      o is Input && shift == o.shift && alt == o.alt && ctrl == o.ctrl;

  int get hashCode => shift.hashCode ^ alt.hashCode ^ ctrl.hashCode;

  String toString() => "{shift: $shift, alt: $alt, ctrl: $ctrl}";
}

class KeyInput extends Input {
  final int keyCode;

  const KeyInput(this.keyCode, {bool shift, bool alt, bool ctrl});

  bool operator ==(Object o) =>
      o is KeyInput && super == o && keyCode == o.keyCode;

  int get hashCode => super.hashCode ^ keyCode.hashCode;

  String toString() => "{keyCode: $keyCode, " + super.toString().substring(1);
}

///Represents an input event generated by a mouse or by touching the display
class PointInput extends Input {
  final int x;
  final int y;

  Vector get position => Vector(x, y);

  const PointInput(this.x, this.y, {bool shift, bool alt, bool ctrl});

  bool operator ==(Object o) =>
      o is PointInput && super == o && x == o.x && y == o.y;

  int get hashCode => super.hashCode ^ x.hashCode ^ y.hashCode;

  String toString() => "{x: $x, y: $y, " + super.toString().substring(1);
}
