import "package:utilary/utilary.dart";
import "sub_terminal.dart";
import "glyph.dart";
import "color.dart";

abstract class Terminal {
  final int rows;
  final int cols;

  int get width => cols;
  int get height => rows;

  Vector get size => Vector(cols, rows);

  Color foreColor = Color.white;
  Color backColor = Color.black;

  Terminal(int width, int height) : rows = height, cols = width;

  void setGlyph(int x, int y, Glyph glyph);

  void drawGlyph(int x, int y, Glyph glyph) => setGlyph(x, y, glyph);

  void drawString(int x, int y, String str, [Color fore, Color back]) {
    Glyph temp;

    fore ??= foreColor;
    back ??= backColor;

    for (int i = 0; i < str.length; i++) {
      temp = Glyph.fromCharCode(str.codeUnitAt(i), fore, back);

      drawGlyph(x + i, y, temp);
    }
  }

  void fill(int x, int y, int width, int height, [Color color]) {
    color ??= backColor;
    Glyph temp = Glyph(" ", foreColor, color);

    for (int i = x; i < width; i++)
      for (int j = y; j < height; j++) drawGlyph(i, j, temp);
  }

  void clear() => fill(0, 0, this.width, this.height, backColor);

  ///Return a SubTerminal, a Terminal inside this Terminal.
  SubTerminal sub(int x, int y, int width, int height){
    if(size.contains(Vector(x, y)) && size.contains(Vector(x + width, y + height)))
      return SubTerminal(x, y, width, height, this);
    
    throw ArgumentError("Can't create a subTerminal that's out of this terminal's bounds");
  }
}
